@@ -83,6 +83,30 @@ sub new {
 		Wx::wxLC_ICON,
 	);
 
+	Wx::Event::EVT_LIST_COL_CLICK(
+		$self,
+		$self->{m_listCtrl1},
+		sub {
+			shift->list_col_click(@_);
+		},
+	);
+
+	Wx::Event::EVT_LIST_ITEM_ACTIVATED(
+		$self,
+		$self->{m_listCtrl1},
+		sub {
+			shift->list_item_activated(@_);
+		},
+	);
+
+	Wx::Event::EVT_LIST_ITEM_SELECTED(
+		$self,
+		$self->{m_listCtrl1},
+		sub {
+			shift->list_item_selected(@_);
+		},
+	);
+
 	$self->{m_htmlWin1} = Wx::HtmlWindow->new(
 		$self,
 		-1,
@@ -91,6 +115,14 @@ sub new {
 		Wx::wxHW_SCROLLBAR_AUTO,
 	);
 
+	$self->{m_checkBox1} = Wx::CheckBox->new(
+		$self,
+		-1,
+		Wx::gettext('Check Me!'),
+		Wx::wxDefaultPosition,
+		Wx::wxDefaultSize,
+	);
+
 	my $bSizer2 = Wx::BoxSizer->new( Wx::wxVERTICAL );
 	$bSizer2->Add( $m_staticText1, 0, Wx::wxALL, 5 );
 	$bSizer2->Add( 10, 5, 1, Wx::wxEXPAND, 5 );
@@ -101,6 +133,7 @@ sub new {
 	$bSizer2->Add( $self->{m_listBox1}, 0, Wx::wxALL, 5 );
 	$bSizer2->Add( $self->{m_listCtrl1}, 0, Wx::wxALL | Wx::wxEXPAND, 5 );
 	$bSizer2->Add( $self->{m_htmlWin1}, 0, Wx::wxALL | Wx::wxEXPAND, 5 );
+	$bSizer2->Add( $self->{m_checkBox1}, 0, Wx::wxALL, 5 );
 
 	my $bSizer1 = Wx::BoxSizer->new( Wx::wxHORIZONTAL );
 	$bSizer1->Add( $bSizer2, 1, Wx::wxEXPAND, 5 );
@@ -112,6 +145,27 @@ sub new {
 	return $self;
 }
 
+sub list_col_click {
+	my $self  = shift;
+	my $event = shift;
+
+	die 'TO BE COMPLETED';
+}
+
+sub list_item_activated {
+	my $self  = shift;
+	my $event = shift;
+
+	die 'TO BE COMPLETED';
+}
+
+sub list_item_selected {
+	my $self  = shift;
+	my $event = shift;
+
+	die 'TO BE COMPLETED';
+}
+
 sub m_button1 {
 	my $self  = shift;
 	my $event = shift;
